/*==============================================================================
    FILE:           user_adau1761.h

    DESCRIPTION:    Provides a interface between the user_audio USB Audio 1.0
                    functionality and the ADI adau1761 driver.

    Copyright (c) 2015 Closed Loop Design, LLC

    This software is supplied "AS IS" without any warranties, express, implied
    or statutory, including but not limited to the implied warranties of fitness
    for purpose, satisfactory quality and non-infringement. Closed Loop Design LLC
    extends you a royalty-free right to reproduce and distribute executable files
    created using this software for use on Analog Devices Blackfin family
    processors only. Nothing else gives you the right to use this software.

==============================================================================*/

#include "user_adau1761.h"
#include <drivers/twi/adi_twi.h>
#include <services/gpio/adi_gpio.h>
#include <services/int/adi_int.h>
#include <stdio.h>
#include <string.h>
#include "SigmaStudio\export\export_IC_1.h"

#include "cld_bf70x_audio_1_0_lib.h"
#include "user_audio.h"

/* the ADAU1761 Rec Mixer Left 0 register */
#define REC_MIX_LEFT_REG                        (0x400A)
/* the ADAU1761 Rec Mixer Right 0 register */
#define REC_MIX_RIGHT_REG                       (0x400C)

/* select sample rate */
#define SAMPLE_RATE                             (ADI_ADAU1761_SAMPLE_RATE_48KHZ)
#define SAMPLE_RATE_IN_HZ                       48000

/* codec device instance to be tested */
#define ADAU1761_DEV_NUM                        0

#define USB_POLLING_INTERVAL_MICROSECONDS       1000
#define NUM_AUDIO_SAMPLES                       (SAMPLE_RATE_IN_HZ/USB_POLLING_INTERVAL_MICROSECONDS)
#define AUDIO_SAMPLE_SIZE_BYTES                 4
#define NUM_AUDIO_CHANNELS                      2

#define USER_ADAU1761_BUFFER_SIZE               (NUM_AUDIO_SAMPLES*AUDIO_SAMPLE_SIZE_BYTES*NUM_AUDIO_CHANNELS)

#pragma align(4)
static unsigned char user_adau1761_sport_rx_mem[ADI_SPORT_DMA_MEMORY_SIZE];

/* Ping Pong buffers used to received Microphone data from the ADAU1761 */
#pragma align(4)
static unsigned char user_adau1761_audio_data_rx_buffer_1[USER_ADAU1761_BUFFER_SIZE];
#pragma align(4)
static unsigned char user_adau1761_audio_data_rx_buffer_2[USER_ADAU1761_BUFFER_SIZE];


static unsigned char *p_available_adau1761_audio_data_rx_buffer;

#pragma align(4)
static unsigned char user_adau1761_sport_tx_mem[ADI_SPORT_DMA_MEMORY_SIZE];

/* Ping Pong buffers used to Transmit Headphone data to the ADAU1761 */
#pragma align(4)
static unsigned char user_adau1761_audio_data_tx_buffer_1[USER_ADAU1761_BUFFER_SIZE];
#pragma align(4)
static unsigned char user_adau1761_audio_data_tx_buffer_2[USER_ADAU1761_BUFFER_SIZE];

/* Pointer to the active Tx audio data ping-pong buffer */
static unsigned char  *p_active_adau1761_audio_data_tx_buffer;
/* Number of bytes in the active Tx ping-pong buffer  */
unsigned short active_adau1761_audio_data_tx_buffer_num_bytes = USER_ADAU1761_BUFFER_SIZE;

/* Memory required for codec driver */
static unsigned char user_adau1761_codec_mem[ADI_ADAU1761_MEMORY_SIZE];

/* adau1761 device handle */
ADI_ADAU1761_HANDLE  hADAU1761;

static CLD_Boolean user_adau_init_error;

ADI_ADAU1761_SPORT_INFO user_adau1761_sport_rx_info;
ADI_ADAU1761_SPORT_INFO user_adau1761_sport_tx_info;

static void user_adau1761_check_adi_result(ADI_ADAU1761_RESULT result);
static void user_adau1761_tx_callback(void *pCBParam, uint32_t Event, void *pArg);
static void user_adau1761_rx_callback(void *pCBParam, uint32_t Event, void *pArg);

/* Queue used to transfer data received over USB to the ADAU1761 Codec */
#define USER_ADAU1761_MAX_NUM_ACTIVE_TRANSFERS      3
typedef struct
{
    unsigned char head;
    unsigned char tail;
    unsigned char free_space;
    User_ADAU1761_Audio_Transfer_Params transfers[USER_ADAU1761_MAX_NUM_ACTIVE_TRANSFERS];
} User_ADAU1761_Audio_Transfer_Queue;

static User_ADAU1761_Audio_Transfer_Queue user_adau1761_tx_audio_transfer_queue =
{
    .head = 0,
    .tail = 0,
    .free_space = USER_ADAU1761_MAX_NUM_ACTIVE_TRANSFERS,
};

/* Flags used to notify mainline if a ping-pong buffer should be resubmitted to the Sport DMA */
static CLD_Boolean user_adau1761_submit_rx_buffer = CLD_FALSE;
static CLD_Boolean user_adau1761_submit_tx_buffer = CLD_FALSE;

/*=============================================================================
Function:       user_adau1761_check_adi_result

Parameters:     result

Description:    Checks if the result passed to the function is an Error.

Returns:        None.
==============================================================================*/
static void user_adau1761_check_adi_result(ADI_ADAU1761_RESULT result)
{
    if (result != ADI_ADAU1761_SUCCESS)
    {
        cld_console(CLD_CONSOLE_RED, CLD_CONSOLE_BLACK, "Codec failure 0x%x\n", result);
        user_adau_init_error = CLD_TRUE;
    }
}

/*=============================================================================
Function:       user_adau1761_init

Parameters:     None

Description:    Initializes the ADAU1761 interface.

Returns:        USER_ADAU1761_INIT_SUCCESS / USER_ADAU1761_INIT_ONGOING/
                USER_ADAU1761_INIT_FAILED.
==============================================================================*/
User_ADAU1761_Init_Return_Code user_adau1761_init (void)
{
    ADI_ADAU1761_RESULT result;
    unsigned char value;

    user_adau_init_error = CLD_FALSE;

    /* Open the codec driver */
    result = adi_adau1761_Open(ADAU1761_DEV_NUM,
            user_adau1761_codec_mem,
            ADI_ADAU1761_MEMORY_SIZE,
            ADI_ADAU1761_COMM_DEV_TWI,
            &hADAU1761 );
    user_adau1761_check_adi_result(result);

    /* Configure TWI */
    result = adi_adau1761_ConfigTWI(hADAU1761, TWI_DEV_NUM, ADI_ADAU1761_TWI_ADDR0);
    user_adau1761_check_adi_result(result);

    /* load Sigma Studio program exported from *_IC_1.h */
    result = adi_adau1761_SigmaStudioLoad(hADAU1761, default_download_IC_1);
    user_adau1761_check_adi_result(result);

    /* config SPORT for Rx data transfer */
    user_adau1761_sport_rx_info.nDeviceNum = SPORT_RX_DEVICE;
    user_adau1761_sport_rx_info.eChannel = ADI_HALF_SPORT_B;
    user_adau1761_sport_rx_info.eMode = ADI_ADAU1761_SPORT_I2S;
    user_adau1761_sport_rx_info.hDevice = NULL;
    user_adau1761_sport_rx_info.pMemory = user_adau1761_sport_rx_mem;
    user_adau1761_sport_rx_info.bEnableDMA = true;
    user_adau1761_sport_rx_info.eDataLen = ADI_ADAU1761_SPORT_DATA_32;
    user_adau1761_sport_rx_info.bEnableStreaming = false;

    result = adi_adau1761_ConfigSPORT (hADAU1761,
            ADI_ADAU1761_SPORT_INPUT, &user_adau1761_sport_rx_info);
    user_adau1761_check_adi_result(result);

    /* register a Rx callback */
    result = adi_adau1761_RegisterRxCallback (hADAU1761,
            user_adau1761_rx_callback, NULL);
    user_adau1761_check_adi_result(result);

    /* config SPORT for Tx data transfer */
    user_adau1761_sport_tx_info.nDeviceNum = SPORT_TX_DEVICE;
    user_adau1761_sport_tx_info.eChannel = ADI_HALF_SPORT_A;
    user_adau1761_sport_tx_info.eMode = ADI_ADAU1761_SPORT_I2S;
    user_adau1761_sport_tx_info.hDevice = NULL;
    user_adau1761_sport_tx_info.pMemory = user_adau1761_sport_tx_mem;
    user_adau1761_sport_tx_info.bEnableDMA = true;
    user_adau1761_sport_tx_info.eDataLen = ADI_ADAU1761_SPORT_DATA_32;
    user_adau1761_sport_tx_info.bEnableStreaming = false;

    result = adi_adau1761_ConfigSPORT (hADAU1761,
            ADI_ADAU1761_SPORT_OUTPUT, &user_adau1761_sport_tx_info);
    user_adau1761_check_adi_result(result);

    /* register a Tx callback */
    result = adi_adau1761_RegisterTxCallback (hADAU1761,
            user_adau1761_tx_callback, NULL);
    user_adau1761_check_adi_result(result);

    /* Select the Digital Microphone input */
    result = adi_adau1761_SelectInputSource(hADAU1761, ADI_ADAU1761_INPUT_MIC);
    user_adau1761_check_adi_result(result);

    /* disable the record mixer (left) */
    result = adi_adau1761_GetRegister (hADAU1761, REC_MIX_LEFT_REG, &value);
    result = adi_adau1761_SetRegister (hADAU1761, REC_MIX_LEFT_REG, value & 0xFE);
    user_adau1761_check_adi_result(result);

    /* disable the record mixer (right) */
    result = adi_adau1761_GetRegister (hADAU1761, REC_MIX_RIGHT_REG, &value);
    result = adi_adau1761_SetRegister (hADAU1761, REC_MIX_RIGHT_REG, value & 0xFE);
    user_adau1761_check_adi_result(result);

    /* Sets the Headphone output volume */
    result = adi_adau1761_SetVolume (hADAU1761,
            ADI_ADAU1761_VOL_HEADPHONE,
            ADI_ADAU1761_VOL_CHAN_BOTH,
            true,
            10);
    user_adau1761_check_adi_result(result);

    /* Mute the Line-Out output */
    result = adi_adau1761_SetVolume (hADAU1761,
            ADI_ADAU1761_VOL_LINE_OUT,
            ADI_ADAU1761_VOL_CHAN_BOTH,
            false,
            0);
    user_adau1761_check_adi_result(result);

    result = adi_adau1761_SetSampleRate (hADAU1761, SAMPLE_RATE);
    user_adau1761_check_adi_result(result);

    /* Sets the Microphone input attenuation */
    result = adi_adau1761_SetVolume (hADAU1761,
            ADI_ADAU1761_VOL_ADC_MIC,
            ADI_ADAU1761_VOL_CHAN_BOTH,
            true,
            0);
    user_adau1761_check_adi_result(result);

    if (user_adau_init_error == CLD_TRUE)
    {
        return USER_ADAU1761_INIT_FAILED;
    }
    else
    {
        return USER_ADAU1761_INIT_SUCCESS;
    }
}

/*=============================================================================
Function:       user_adau1761_main

Parameters:     None

Description:    ADAU1761 mainline function.

Returns:        None.
==============================================================================*/
void user_adau1761_main (void)
{
    ADI_ADAU1761_RESULT result;

    /* re-submit processed Rx buffer to the Sport DMA */
    if (user_adau1761_submit_rx_buffer == CLD_TRUE)
    {
        user_adau1761_submit_rx_buffer = CLD_FALSE;
        if (p_available_adau1761_audio_data_rx_buffer != NULL)
        {
            result = adi_adau1761_SubmitRxBuffer(hADAU1761, p_available_adau1761_audio_data_rx_buffer, USER_ADAU1761_BUFFER_SIZE);
            user_adau1761_check_adi_result(result);
        }
    }
    /* re-submit processed Tx buffer to the Sport DMA */
    if (user_adau1761_submit_tx_buffer == CLD_TRUE)
    {
        user_adau1761_submit_tx_buffer = CLD_FALSE;
        if (p_active_adau1761_audio_data_tx_buffer != NULL)
        {
            result = adi_adau1761_SubmitTxBuffer(hADAU1761, p_active_adau1761_audio_data_tx_buffer, active_adau1761_audio_data_tx_buffer_num_bytes);
            user_adau1761_check_adi_result(result);
        }
    }
}

/*=============================================================================
Function:       user_adau1761_rx_callback

Parameters:     Event - Identifies why the callback was called by the Sport driver.
                pArg - Points to the starting address of the buffer that was processed.

Description:    Sport RX DMA callback function.

Returns:        None.
==============================================================================*/
static void user_adau1761_rx_callback(void *pCBParam, uint32_t Event, void *pArg)
{
    /* If Microphone data has been received. */
    if (Event == (unsigned long)ADI_ADAU1761_EVENT_RX_BUFFER_PROCESSED)
    {
        /* Flag that the buffer is ready to be resubmitted */
        p_available_adau1761_audio_data_rx_buffer = pArg;
        user_adau1761_submit_rx_buffer = CLD_TRUE;

        /* Pass the received Microphone data to the USB Audio. */
        user_audio_transmit_audio_stream_data (pArg, USER_ADAU1761_BUFFER_SIZE);
    }
}

/*=============================================================================
Function:       user_adau1761_tx_callback

Parameters:     Event - Identifies why the callback was called by the Sport driver.
                pArg - Points to the starting address of the buffer that was processed.

Description:    Sport Tx DMA callback function.

Returns:        None.
==============================================================================*/
static void user_adau1761_tx_callback(void *pCBParam, uint32_t Event, void *pArg)
{
    /* If USB audio data has been transmitted to the ADAU1761 codec. */
    if (Event == (unsigned long)ADI_ADAU1761_EVENT_TX_BUFFER_PROCESSED)
    {
        /* Flag that the buffer is ready to be resubmitted */
        p_active_adau1761_audio_data_tx_buffer = pArg;
        user_adau1761_submit_tx_buffer = CLD_TRUE;

        /* If there is audio data waiting to be sent to the ADAU1761 codec */
        if (user_adau1761_tx_audio_transfer_queue.free_space != USER_ADAU1761_MAX_NUM_ACTIVE_TRANSFERS)
        {
            /* Copy the data from the queue to the ping-pong buffer that was just processed. */
            memcpy(pArg, user_adau1761_tx_audio_transfer_queue.transfers[user_adau1761_tx_audio_transfer_queue.tail].p_buffer, user_adau1761_tx_audio_transfer_queue.transfers[user_adau1761_tx_audio_transfer_queue.tail].buffer_size);
            /* Save the number of bytes that were loaded into the buffer. */
            active_adau1761_audio_data_tx_buffer_num_bytes = user_adau1761_tx_audio_transfer_queue.transfers[user_adau1761_tx_audio_transfer_queue.tail].buffer_size;

            /* Call the fp_xfr_complete callback to let the requester know that the audio data is being transmitted */
            if (user_adau1761_tx_audio_transfer_queue.transfers[user_adau1761_tx_audio_transfer_queue.tail].fp_xfr_complete)
            {
                user_adau1761_tx_audio_transfer_queue.transfers[user_adau1761_tx_audio_transfer_queue.tail].fp_xfr_complete();
            }
            user_adau1761_tx_audio_transfer_queue.tail++;
            if (user_adau1761_tx_audio_transfer_queue.tail >= USER_ADAU1761_MAX_NUM_ACTIVE_TRANSFERS)
            {
                user_adau1761_tx_audio_transfer_queue.tail = 0;
            }
            user_adau1761_tx_audio_transfer_queue.free_space++;
        }
    }
}

/*=============================================================================
Function:       user_adau1761_SetVolume

Parameters:     hDevice     - ADAU1761 ADI device handle
                eVolumeCtrl - identifies which ADAU1761 volume control is
                              being modified.
                eChannel    - Identifies which audio channel is being modified
                usb_volume  - requested USB Audio v1.0 volume level.

Description:    Sets the ADAU1761 Volume (or attenuation) for the specified
                control and channel.

Returns:        ADI_ADAU1761_RESULT.
==============================================================================*/
ADI_ADAU1761_RESULT user_adau1761_SetVolume (
    ADI_ADAU1761_HANDLE            const hDevice,
    ADI_ADAU1761_VOLUME_CONTROL    const eVolumeCtrl,
    ADI_ADAU1761_VOLUME_CHANNEL    const eChannel,
    unsigned short                 const usb_volume)
{
    unsigned char mute;
    ADI_ADAU1761_RESULT result;

    /* If setting the headphone volume */
    if (eVolumeCtrl == ADI_ADAU1761_VOL_HEADPHONE)
    {
        /* Get the current Mute value for the headphones.
           The value returned by user_adau1761_GetMute matches the expected value for
           the USB Audio v1.0 protocol, which is inverted from the ADAU1761 protocol
           so invert the returned value before sending it to the codec. */
        mute = !user_adau1761_GetMute(hDevice, eVolumeCtrl, eChannel);
        /* Convert the requested USB volume to the Codec volume and call the
           ADI set volume function. */
        result = adi_adau1761_SetVolume (hDevice, eVolumeCtrl, eChannel, mute, user_adau1761_convert_usb_volume_to_codec(usb_volume));
    }
    /* If setting the microphone attenuation */
    else if (eVolumeCtrl == ADI_ADAU1761_VOL_ADC_MIC)
    {
        /* Convert the requested USB volume to the Codec volume and call the
           ADI set volume function. */
        result = adi_adau1761_SetVolume (hDevice, eVolumeCtrl, eChannel, true, user_adau1761_convert_usb_volume_to_codec_mic(usb_volume));
    }
    user_adau1761_check_adi_result(result);

    return result;
}

/*=============================================================================
Function:       user_adau1761_SetMute

Parameters:     hDevice     - ADAU1761 ADI device handle
                eVolumeCtrl - identifies which ADAU1761 volume control is
                              being modified.
                Mute        - 1 = Mute, 0 = Un-mute.

Description:    Mutes the requested ADAU1761 channel.

Returns:        ADI_ADAU1761_RESULT.
==============================================================================*/
ADI_ADAU1761_RESULT user_adau1761_SetMute (
    ADI_ADAU1761_HANDLE            const hDevice,
    ADI_ADAU1761_VOLUME_CONTROL    const eVolumeCtrl,
    unsigned char                  const mute)
{
    uint16_t regLeftAddr;
    uint16_t regRightAddr;
    unsigned char reg_value;
    ADI_ADAU1761_RESULT result = ADI_ADAU1761_SUCCESS;

    switch(eVolumeCtrl)
    {
        case ADI_ADAU1761_VOL_HEADPHONE:
            regLeftAddr = REG_HEADPHONE_LEFT_ADDR;
            regRightAddr = REG_HEADPHONE_RIGHT_ADDR;
        break;
        case ADI_ADAU1761_VOL_LINE_OUT:
            regLeftAddr = REG_LINE_OUT_LEFT_ADDR;
            regRightAddr = REG_LINE_OUT_RIGHT_ADDR;
        break;
        case ADI_ADAU1761_VOL_ADC_MIC:
            regLeftAddr = REG_ADC_MIC_LEFT_ADDR;
            regRightAddr = REG_ADC_MIC_RIGHT_ADDR;
        break;
        case ADI_ADAU1761_VOL_DAC:
            regLeftAddr = REG_DAC_LEFT_ADDR;
            regRightAddr = REG_DAC_RIGHT_ADDR;
        break;
        case ADI_ADAU1761_VOL_LINE_IN_DIFF:
            regLeftAddr = REG_DIFF_IN_LEFT_ADDR;
            regRightAddr = REG_DIFF_IN_RIGHT_ADDR;
        break;
        default:
            result = ADI_ADAU1761_INVALID_OPERATION;
        break;
    }

    if (result == ADI_ADAU1761_SUCCESS)
    {
        /* Read the current volume register setting for the left channel */
        result = adi_adau1761_GetRegister (hDevice, regLeftAddr, &reg_value);
        /* Set or clear the mute flag */
        if (mute)
        {
            reg_value &= ~0x02;
        }
        else
        {
            reg_value |= 0x02;
        }
        result = adi_adau1761_SetRegister (hDevice, regLeftAddr, reg_value);

        /* Read the current volume register setting for the right channel */
        result = adi_adau1761_GetRegister (hDevice, regRightAddr, &reg_value);
        if (mute)
        {
            reg_value &= ~0x02;
        }
        else
        {
            reg_value |= 0x02;
        }
        result = adi_adau1761_SetRegister (hDevice, regRightAddr, reg_value);
    }

    return result;
}

/*=============================================================================
Function:       user_adau1761_GetVolume

Parameters:     hDevice     - ADAU1761 ADI device handle
                eVolumeCtrl - identifies which ADAU1761 volume control is
                              being read.
                eChannel    - Identifies which audio channel is being read
                p_volume    - Pointer to the buffer to store the current
                              volume (USB Audio v1.0 format).

Description:    Gets the ADAU1761 Volume (or attenuation) for the specified
                control and channel.

Returns:        ADI_ADAU1761_RESULT.
==============================================================================*/
ADI_ADAU1761_RESULT user_adau1761_GetVolume (
    ADI_ADAU1761_HANDLE            const hDevice,
    ADI_ADAU1761_VOLUME_CONTROL    const eVolumeCtrl,
    ADI_ADAU1761_VOLUME_CHANNEL    const eChannel,
    unsigned short *                p_volume)
{
    uint16_t regLeftAddr;
    uint16_t regRightAddr;
    ADI_ADAU1761_RESULT result = ADI_ADAU1761_SUCCESS;

    unsigned char volume;

    switch(eVolumeCtrl)
    {
        case ADI_ADAU1761_VOL_HEADPHONE:
            regLeftAddr = REG_HEADPHONE_LEFT_ADDR;
            regRightAddr = REG_HEADPHONE_RIGHT_ADDR;
        break;
        case ADI_ADAU1761_VOL_LINE_OUT:
            regLeftAddr = REG_LINE_OUT_LEFT_ADDR;
            regRightAddr = REG_LINE_OUT_RIGHT_ADDR;
        break;
        case ADI_ADAU1761_VOL_ADC_MIC:
            regLeftAddr = REG_ADC_MIC_LEFT_ADDR;
            regRightAddr = REG_ADC_MIC_RIGHT_ADDR;
        break;
        case ADI_ADAU1761_VOL_DAC:
            regLeftAddr = REG_DAC_LEFT_ADDR;
            regRightAddr = REG_DAC_RIGHT_ADDR;
        break;
        case ADI_ADAU1761_VOL_LINE_IN_DIFF:
            regLeftAddr = REG_DIFF_IN_LEFT_ADDR;
            regRightAddr = REG_DIFF_IN_RIGHT_ADDR;
        break;
        default:
            result = ADI_ADAU1761_INVALID_OPERATION;
        break;
    }

    if (result == ADI_ADAU1761_SUCCESS)
    {
        if ((eChannel == ADI_ADAU1761_VOL_CHAN_LEFT) ||
            (eChannel == ADI_ADAU1761_VOL_CHAN_BOTH))
        {
            result = adi_adau1761_GetRegister (hDevice, regLeftAddr, &volume);
        }

        else if ((eChannel == ADI_ADAU1761_VOL_CHAN_RIGHT) ||
            (eChannel == ADI_ADAU1761_VOL_CHAN_BOTH))
        {
            result = adi_adau1761_GetRegister (hDevice, regRightAddr, &volume);
        }
    }
    if (eVolumeCtrl == ADI_ADAU1761_VOL_HEADPHONE)
    {
        volume >>= 2;

        *p_volume = user_adau1761_convert_1761_volume_to_usb(volume);
    }
    else if (eVolumeCtrl == ADI_ADAU1761_VOL_ADC_MIC)
    {
        *p_volume = user_adau1761_convert_mic_1761_volume_to_usb(volume);
    }

    user_adau1761_check_adi_result(result);

    return result;
}

/*=============================================================================
Function:       user_adau1761_GetMute

Parameters:     hDevice     - ADAU1761 ADI device handle
                eVolumeCtrl - identifies which ADAU1761 volume control is
                              being read.
                eChannel    - Identifies which audio channel is being read

Description:    Gets the ADAU1761 Mute setting for the specified
                control and channel.

Returns:        0x01 if Muted, 0x00 if no muted.
==============================================================================*/
unsigned char user_adau1761_GetMute (
    ADI_ADAU1761_HANDLE            const hDevice,
    ADI_ADAU1761_VOLUME_CONTROL    const eVolumeCtrl,
    ADI_ADAU1761_VOLUME_CHANNEL    const eChannel)
{
    uint16_t regLeftAddr;
    uint16_t regRightAddr;
    ADI_ADAU1761_RESULT result = ADI_ADAU1761_SUCCESS;

    unsigned char mute;

    switch(eVolumeCtrl)
    {
        case ADI_ADAU1761_VOL_HEADPHONE:
            regLeftAddr = REG_HEADPHONE_LEFT_ADDR;
            regRightAddr = REG_HEADPHONE_RIGHT_ADDR;
        break;
        case ADI_ADAU1761_VOL_LINE_OUT:
            regLeftAddr = REG_LINE_OUT_LEFT_ADDR;
            regRightAddr = REG_LINE_OUT_RIGHT_ADDR;
        break;
        case ADI_ADAU1761_VOL_ADC_MIC:
            regLeftAddr = REG_ADC_MIC_LEFT_ADDR;
            regRightAddr = REG_ADC_MIC_RIGHT_ADDR;
        break;
        case ADI_ADAU1761_VOL_DAC:
            regLeftAddr = REG_DAC_LEFT_ADDR;
            regRightAddr = REG_DAC_RIGHT_ADDR;
        break;
        case ADI_ADAU1761_VOL_LINE_IN_DIFF:
            regLeftAddr = REG_DIFF_IN_LEFT_ADDR;
            regRightAddr = REG_DIFF_IN_RIGHT_ADDR;
        break;
        default:
            result = ADI_ADAU1761_INVALID_OPERATION;
        break;
    }

    if (result == ADI_ADAU1761_SUCCESS)
    {
        if ((eChannel == ADI_ADAU1761_VOL_CHAN_LEFT) ||
            (eChannel == ADI_ADAU1761_VOL_CHAN_BOTH))
        {
            result = adi_adau1761_GetRegister (hDevice, regLeftAddr, &mute);
        }

        else if ((eChannel == ADI_ADAU1761_VOL_CHAN_RIGHT) ||
            (eChannel == ADI_ADAU1761_VOL_CHAN_BOTH))
        {
            result = adi_adau1761_GetRegister (hDevice, regRightAddr, &mute);
        }
    }
    /* Bit 1 of the ADAU1761 volume register is the Mute setting */
    mute = ((~mute) >> 1) & 0x01;
    user_adau1761_check_adi_result(result);

    return mute;
}

/*=============================================================================
Function:       user_adau1761_convert_1761_volume_to_usb

Parameters:     adau1761_vol - ADAU1761 volume setting

Description:    Converts the passed ADAU1761 headphone volume setting to the equivalent
                USB Audio v1.0 setting.

                The ADAU1761 volume is a 6-bit value that ranges from -57dB to 6dB
                in 1dB increments.
                The USB volume is a 8-bit value ranging from -127.
                ranges from -127.9961 to 127.9961 in 1/256 dB increments

Returns:        USB Audio v1.0 volume setting.
==============================================================================*/
unsigned short user_adau1761_convert_1761_volume_to_usb (unsigned char adau1761_vol)
{
    unsigned short usb_vol;

    if (adau1761_vol < 0x39)
    {
        /* negative db */
        usb_vol = ((0xff - (0x39-adau1761_vol))+1) << 8;
    }
    else
    {
        usb_vol = (adau1761_vol - 0x39) << 8;
    }
    return usb_vol;
}

/*=============================================================================
Function:       user_adau1761_convert_mic_1761_volume_to_usb

Parameters:     adau1761_vol - ADAU1761 volume setting

Description:    Converts the passed ADAU1761 microphone attenuation setting to
                the equivalent USB Audio v1.0 setting.

                The ADAU1761 volume is a 8-bit value that ranges from -95.625dB to 0dB
                in 0.375dB increments.
                The USB volume is a 8-bit value ranging from -127.
                ranges from -127.9961 to 127.9961 in 1/256 dB increments

Returns:        USB Audio v1.0 volume setting.
==============================================================================*/
unsigned short user_adau1761_convert_mic_1761_volume_to_usb (unsigned char adau1761_vol)
{
    unsigned short usb_vol;

    if (adau1761_vol == 0)
    {
        usb_vol = 0;
    }
    else
    {
        usb_vol = (adau1761_vol * 96) | 0x8000;
    }

    return usb_vol;
}

/*=============================================================================
Function:       user_adau1761_convert_usb_volume_to_codec

Parameters:     usb_vol - USB Audio volume setting

Description:    Converts the passed USB audio headphone volume setting to the equivalent
                ADAU1761 setting.

                The ADAU1761 volume is a 6-bit value that ranges from -57dB to 6dB
                in 1dB increments.
                The USB volume is a 8-bit value ranging from -127.
                ranges from -127.9961 to 127.9961 in 1/256 dB increments

Returns:        ADAU1761 volume setting.
==============================================================================*/
unsigned char user_adau1761_convert_usb_volume_to_codec (unsigned short usb_vol)
{
    signed char adj_usb;

    adj_usb = (signed char)(usb_vol >> 8);

    if (adj_usb < 0)
    {
        adj_usb++;
        if (adj_usb < -57)
        {
            adj_usb = -57;
        }
        adj_usb += 57;
    }
    else
    {
        if (adj_usb > 6)
        {
            adj_usb = 6;
        }
        adj_usb += 0x39;
    }
    return (unsigned char)adj_usb;
}


/*=============================================================================
Function:       user_adau1761_convert_usb_volume_to_codec_mic

Parameters:     usb_vol - USB Audio volume setting

Description:    Converts the passed USB microphone attenuation setting to
                the equivalent ADAU1761 setting.

                The ADAU1761 volume is a 8-bit value that ranges from -95.625dB to 0dB
                in 0.375dB increments.
                The USB volume is a 8-bit value ranging from -127.
                ranges from -127.9961 to 127.9961 in 1/256 dB increments

Returns:        ADAU1761 attenuation setting.
==============================================================================*/
unsigned char user_adau1761_convert_usb_volume_to_codec_mic (unsigned short usb_vol)
{
    signed short adj_usb;
    unsigned char codec_vol;

    if (usb_vol == 0)
    {
        codec_vol = 0;
    }
    else
    {
        adj_usb = (signed short)usb_vol;
        adj_usb *= -1;
        codec_vol = (adj_usb / 96);
    }
    return codec_vol;
}

/*=============================================================================
Function:       user_adau1761_transfer_audio_buffer

Parameters:     p_params - describes a Headphone audio data buffer

Description:    Adds the specified audio data buffer to the Headphone circular
                queue.

Returns:        None.
==============================================================================*/
void user_adau1761_transfer_audio_buffer (User_ADAU1761_Audio_Transfer_Params * p_params)
{
    if (user_adau1761_tx_audio_transfer_queue.free_space)
    {
        user_adau1761_tx_audio_transfer_queue.transfers[user_adau1761_tx_audio_transfer_queue.head] = *p_params;
        user_adau1761_tx_audio_transfer_queue.head++;
        if (user_adau1761_tx_audio_transfer_queue.head >= USER_ADAU1761_MAX_NUM_ACTIVE_TRANSFERS)
        {
            user_adau1761_tx_audio_transfer_queue.head = 0;
        }
        user_adau1761_tx_audio_transfer_queue.free_space--;
    }
}

/*=============================================================================
Function:       user_adau1761_enable_output

Parameters:     None.

Description:    Enables the ADAU1761 headphone output and submits the two
                tx ping-pong buffers to the Sport driver.

Returns:        None.
==============================================================================*/
void user_adau1761_enable_output (void)
{
    ADI_ADAU1761_RESULT result = ADI_ADAU1761_SUCCESS;
    /* Disable audio output */
    result = adi_adau1761_EnableOutput (hADAU1761, false);
    user_adau1761_check_adi_result(result);

    /* submit buffer1 */
    result = adi_adau1761_SubmitTxBuffer(hADAU1761, user_adau1761_audio_data_tx_buffer_1, USER_ADAU1761_BUFFER_SIZE);
    user_adau1761_check_adi_result(result);

    /* submit buffer2 */
    result = adi_adau1761_SubmitTxBuffer(hADAU1761, user_adau1761_audio_data_tx_buffer_2, USER_ADAU1761_BUFFER_SIZE);
    user_adau1761_check_adi_result(result);

    /* Enable audio output */
    result = adi_adau1761_EnableOutput (hADAU1761, true);
    user_adau1761_check_adi_result(result);
}

/*=============================================================================
Function:       user_adau1761_disable_output

Parameters:     None.

Description:    Disables the ADAU1761 audio output.

Returns:        None.
==============================================================================*/
void user_adau1761_disable_output (void)
{
    ADI_ADAU1761_RESULT result = ADI_ADAU1761_SUCCESS;

    result = adi_adau1761_EnableOutput (hADAU1761, false);
    user_adau1761_check_adi_result(result);
}

/*=============================================================================
Function:       user_adau1761_enable_input

Parameters:     None.

Description:    Enables the ADAU1761 microphone input and submits the two
                Rx ping-pong buffers to the Sport driver.

Returns:        None.
==============================================================================*/
void user_adau1761_enable_input (void)
{
    ADI_ADAU1761_RESULT result;

    /* Disable microphone input */
    result = adi_adau1761_EnableInput (hADAU1761, false);
    user_adau1761_check_adi_result(result);

    /* submit buffer1 */
    result = adi_adau1761_SubmitRxBuffer(hADAU1761, user_adau1761_audio_data_rx_buffer_1, USER_ADAU1761_BUFFER_SIZE);
    user_adau1761_check_adi_result(result);

    /* submit buffer2 */
    result = adi_adau1761_SubmitRxBuffer(hADAU1761, user_adau1761_audio_data_rx_buffer_2, USER_ADAU1761_BUFFER_SIZE);
    user_adau1761_check_adi_result(result);

    /* Enable microphone input */
    result = adi_adau1761_EnableInput (hADAU1761, true);
    user_adau1761_check_adi_result(result);
}

/*=============================================================================
Function:       user_adau1761_disable_input

Parameters:     None.

Description:    Disables the ADAU1761 microphone input.

Returns:        None.
==============================================================================*/
void user_adau1761_disable_input (void)
{
    ADI_ADAU1761_RESULT result;
    result = adi_adau1761_EnableInput (hADAU1761, false);
    user_adau1761_check_adi_result(result);
}
