/*==============================================================================
    FILE:           user_audio.c

    DESCRIPTION:    Uses the cld_bf70x_audio_1_0_lib library to implement a basic
                    USB audio device.

    Copyright (c) 2015 Closed Loop Design, LLC

    This software is supplied "AS IS" without any warranties, express, implied
    or statutory, including but not limited to the implied warranties of fitness
    for purpose, satisfactory quality and non-infringement. Closed Loop Design LLC
    extends you a royalty-free right to reproduce and distribute executable files
    created using this software for use on Analog Devices Blackfin family
    processors only. Nothing else gives you the right to use this software.

==============================================================================*/
#include "user_audio.h"
#include "cld_bf70x_audio_1_0_lib.h"

#include "user_adau1761.h"
#include <ADSP-BF707_device.h>
#include <string.h>

/* Feature Unit ID used to control the Headphone volume */
#define USER_AUDIO_HEADPHONE_FEATURE_UNIT_ID                    0x09
/* Control and Channel number from the Setup Packet wValue field of
   the Headphone left channel */
#define USER_AUDIO_HEADPHONE_CONTROL_AND_CHANNEL_NUM_LEFT       0x0201
/* Control and Channel number from the Setup Packet wValue field of
   the Headphone right channel */
#define USER_AUDIO_HEADPHONE_CONTROL_AND_CHANNEL_NUM_RIGHT      0x0202

/* Feature Unit ID used to control the Microphone volume */
#define USER_AUDIO_MICROPHONE_FEATURE_UNIT_ID                   0x0a
/* Control and Channel number from the Setup Packet wValue field of
   the Microphone volue */
#define USER_AUDIO_MICROPHONE_CONTROL_AND_CHANNEL_NUM_MASTER    0x0200

/* High Speed max packet size should be large enough to hold 1 bInterval worth
   of audio samples.
   With a sample rate of 48Khz, 32-bit resolution (4 bytes) and a bInterval of 1 millisecond
   The max packet size should be >= 384 bytes

   48kHz = 48 samples per Millisecond
   48 * 4 bytes per channel * 2 channels = 384 */
#define USER_AUDIO_MAX_PACKET_SIZE                              400

#pragma pack (1)

/* USB Audio v1.0 Unit and Terminal descriptors that describe a simple
   audio device comprised of the following:

    Input Terminal - USB Streaming Endpoint
        ID = 0x01
        Channels: Left, Right
    Input Terminal - Microphone
        ID = 0x02
        Channels: Left, Right
    Output Terminal - Speaker
        ID = 0x06
        Source ID = 0x09
    Output Terminal - USB Streaming Endpoint
        ID = 0x07
        Source ID = 0x0a
    Feature Unit
        ID = 0x09
        Source ID = 0x01
        Controls:
            Master Channel 0: Mute (Control 1)
            Channel 1 (Left): Volume (Control 2)
            Channel 2 (Right): Volume (Control 2)
    Feature Unit
        ID = 0x0a
        Source ID = 0x02
        Controls:
            Master Channel 0: Volume (Control 2)
    */
static const unsigned char user_audio_unit_and_terminal_descriptor[] =
{
    /* Input Terminal Descriptor - USB Endpoint */
    0x0C,               /* bLength */
    0x24,               /* bDescriptorType = Class Specific Interface */
    0x02,               /* bDescriptorSubType = Input Terminal */
    0x01,               /* bTerminalID */
    0x01, 0x01,         /* wTerminalType = USB Streaming */
    0x00,               /* bAssocTerminal */
    0x02,               /* bNRChannels */
    0x03, 0x00,         /* wChannelConfig (Left & Right Present) */
    0x00,               /* iChannelConfig */
    0x00,               /* iTerminal */
    /* Input Terminal Descriptor - Microphone */
    0x0C,               /* bLength */
    0x24,               /* bDescriptorType = Class Specific Interface */
    0x02,               /* bDescriptorSubType = Input Terminal */
    0x02,               /* bTerminalID */
    0x01, 0x02,         /* wTerminalType = Microphone */
    0x00,               /* bAssocTerminal */
    0x02,               /* bNRChannels */
    0x03, 0x00,         /* wChannelConfig (Left & Right Present) */
    0x00,               /* iChannelConfig */
    0x00,               /* iTerminal */
    /* Output Terminal Descriptor - Speaker */
    0x09,               /* bLength */
    0x24,               /* bDescriptorType = Class Specific Interface */
    0x03,               /* bDescriptorSubType = Output Terminal */
    0x06,               /* bTerminalID */
    0x01, 0x03,         /* wTerminalType - Speaker */
    0x00,               /* bAssocTerminal */
    0x09,               /* bSourceID */
    0x00,               /* iTerminal */
    /* Output Terminal Descriptor - USB Endpoint */
    0x09,               /* bLength */
    0x24,               /* bDescriptorType = Class Specific Interface */
    0x03,               /* bDescriptorSubType = Output Terminal */
    0x07,               /* bTerminalID */
    0x01, 0x01,         /* wTerminalType - USB Streaming */
    0x00,               /* bAssocTerminal */
    0x0a,               /* bSourceID */
    0x00,               /* iTerminal */
    /* Feature Unit Descriptor */
    0x0A,               /* bLength */
    0x24,               /* bDescriptorType = Class Specific Interface */
    0x06,               /* bDescriptorSubType = Feature Unit */
    0x09,               /* bUnitID */
    0x01,               /* bSourceID */
    0x01,               /* bControlSize */
    0x01,               /* mbaControls(0) - Mute Supported */
    0x02,               /* mbaControls(1) - Volume Supported */
    0x02,               /* mbaControls(2) - Volume Supported */
    0x00,               /* iFeature */
    /* Feature Unit Descriptor */
    0x0A,               /* bLength */
    0x24,               /* bDescriptorType = Class Specific Interface */
    0x06,               /* bDescriptorSubType = Feature Unit */
    0x0A,               /* bUnitID */
    0x02,               /* bSourceID */
    0x01,               /* bControlSize */
    0x02,               /* mbaControls(0) - Volume */
    0x00,               /* mbaControls(1) */
    0x00,               /* mbaControls(2) */
    0x00,               /* iFeature */
};

/* Isochronous IN endpoint PCM format descriptor */
static const unsigned char user_audio_in_stream_format_descriptor[] =
{
    0x0b,               /* bLength */
    0x24,               /* bDescriptorType - Class Specific Interface */
    0x02,               /* bDescriptorSubType - Format Type */
    0x01,               /* bFormatType - Format Type 1 */
    0x02,               /* bNrChannels */
    0x04,               /* bSubFrameSize */
    0x20,               /* bBitResolution */
    0x01,               /* bSamFreqType */
    0x80, 0xBB, 0x00,   /* tSamFreq(1) = 48.0Khz */
};

/* Isochronous OUT endpoint PCM format descriptor */
static const unsigned char user_audio_out_stream_format_descriptor[] =
{
    0x0b,               /* bLength */
    0x24,               /* bDescriptorType - Class Specific Interface */
    0x02,               /* bDescriptorSubType - Format Type */
    0x01,               /* bFormatType - Format Type 1 */
    0x02,               /* bNrChannels */
    0x04,               /* bSubFrameSize */
    0x20,               /* bBitResolution */
    0x01,               /* bSamFreqType */
    0x80, 0xBB, 0x00,   /* tSamFreq(1) = 48.0Khz */
};


#pragma pack ()

/* IN Audio Stream Interface Endpoint Data Descriptor */
static const CLD_BF70x_Audio_1_0_Lib_Audio_Stream_Data_Endpoint_Descriptor user_audio_in_stream_endpoint_desc =
{
    .b_length = sizeof(CLD_BF70x_Audio_1_0_Lib_Audio_Stream_Data_Endpoint_Descriptor),
    .b_descriptor_type                  = 0x25,             /* Class Specific Endpoint */
    .b_descriptor_subtype               = 0x01,             /* Endpoint - General */
    .bm_attributes                      = 0x01,             /* sampling freq supported */
    .b_lock_delay_units                 = 0x00,             /* Undefined */
    .w_lock_delay                       = 0x00,
};

static const CLD_BF70x_Audio_1_0_Lib_Audio_Stream_Data_Endpoint_Descriptor user_audio_out_stream_endpoint_desc =
{
    .b_length = sizeof(CLD_BF70x_Audio_1_0_Lib_Audio_Stream_Data_Endpoint_Descriptor),
    .b_descriptor_type                  = 0x25,             /* Class Specific Endpoint */
    .b_descriptor_subtype               = 0x01,             /* Endpoint - General */
    .bm_attributes                      = 0x01,             /* sampling freq supported */
    .b_lock_delay_units                 = 0x01,             /* Milliseconds */
    .w_lock_delay                       = 0x01,             /* 1 Millisecond */
};


/* Function prototypes */
static void user_audio_audio_stream_data_transmitted (void);
static void user_audio_audio_stream_data_transmit_aborted (void);
static CLD_USB_Transfer_Request_Return_Type user_audio_stream_data_received (CLD_USB_Transfer_Params * p_transfer_data);
static CLD_USB_Data_Received_Return_Type user_audio_stream_data_receive_complete (void);
static void user_audio_adau1761_buffer_transmitted (void);
static CLD_USB_Transfer_Request_Return_Type user_audio_set_req_cmd (CLD_BF70x_Audio_1_0_Cmd_Req_Parameters * p_req_params, CLD_USB_Transfer_Params * p_transfer_data);
static CLD_USB_Transfer_Request_Return_Type user_audio_get_req_cmd (CLD_BF70x_Audio_1_0_Cmd_Req_Parameters * p_req_params, CLD_USB_Transfer_Params * p_transfer_data);
static CLD_USB_Data_Received_Return_Type user_audio_set_volume_req (void);
static void user_audio_streaming_rx_endpoint_enabled (CLD_Boolean enabled);
static void user_audio_streaming_tx_endpoint_enabled (CLD_Boolean enabled);
static void user_audio_console_rx_byte (unsigned char byte);
static void user_audio_usb_event (CLD_USB_Event event);

/* Audio Stream IN Interface parameters */
static CLD_BF70x_Audio_1_0_Stream_Interface_Params user_audio_in_endpoint_params =
{
    .endpoint_number            = 1,                        /* Isochronous endpoint number */
    .max_packet_size_full_speed = USER_AUDIO_MAX_PACKET_SIZE,/* Isochronous endpoint full-speed max packet size */
    .max_packet_size_high_speed = USER_AUDIO_MAX_PACKET_SIZE,/* Isochronous endpoint high-speed max packet size */
    .b_interval_full_speed      = 1,                        /* Isochronous endpoint full-speed bInterval */
    .b_interval_high_speed      = 4,                        /* Isochronous endpoint high-speed bInterval - 1 millisecond */
    .synchronization_type       = 0x1,                      /* Isochronous endpoint synchronization type = Asynchronous */
    .b_terminal_link            = 7,                        /* Terminal ID of the associated Output Terminal */
    .b_delay                    = 1,                        /* Delay = 1 Frame */
    .w_format_tag               = 1,                        /* PCM */
                                                            /* Pointer to the PCM Format Descriptor */
    .p_format_type_descriptor   = (unsigned char*)user_audio_in_stream_format_descriptor,
    .p_audio_stream_endpoint_data_descriptor = (CLD_BF70x_Audio_1_0_Lib_Audio_Stream_Data_Endpoint_Descriptor*)&user_audio_in_stream_endpoint_desc,
};

/* Audio Stream OUT Interface parameters */
static CLD_BF70x_Audio_1_0_Stream_Interface_Params user_audio_out_endpoint_params =
{
    .endpoint_number            = 1,                        /* Isochronous endpoint number */
    .max_packet_size_full_speed = USER_AUDIO_MAX_PACKET_SIZE,/* Isochronous endpoint full-speed max packet size */
    .max_packet_size_high_speed = USER_AUDIO_MAX_PACKET_SIZE,/* Isochronous endpoint high-speed max packet size */
    .b_interval_full_speed      = 1,                        /* Isochronous endpoint full-speed bInterval */
    .b_interval_high_speed      = 4,                        /* Isochronous endpoint high-speed bInterval - 1 millisecond */
    .synchronization_type       = 0x2,                      /* Isochronous endpoint synchronization type = Adaptive */
    .b_terminal_link            = 1,                        /* Terminal ID of the associated Output Terminal */
    .b_delay                    = 1,                        /* Delay = 1 Frame */
    .w_format_tag               = 1,                        /* PCM */
                                                            /* Pointer to the PCM Format Descriptor */
    .p_format_type_descriptor        = (unsigned char*)user_audio_out_stream_format_descriptor,
    .p_audio_stream_endpoint_data_descriptor = (CLD_BF70x_Audio_1_0_Lib_Audio_Stream_Data_Endpoint_Descriptor*)&user_audio_out_stream_endpoint_desc,
};

/* CLD BF70x Audio 1.0 library initialization data. */
static CLD_BF70x_Audio_1_0_Lib_Init_Params user_audio_init_params =
{
    .timer_num  = CLD_TIMER_0,                          /* Timer used by the CLD Library */
    .uart_num   = CLD_UART_0,                           /* UART used by the CLD Library.
                                                            If the uart_num is set to
                                                            CLD_UART_DISABLE the CLD library
                                                            will not use a UART */
    .uart_baud  = 115200,                               /* CLD Library CONSOLE print UART
                                                           baudrate. */
    .sclk0      = 100000000u,                           /* Blackfin SCLK0 frequency */
    .fp_console_rx_byte = user_audio_console_rx_byte,    /* Function called when a byte
                                                           is received by the CONSOLE
                                                           UART. */
    .vendor_id = 0x064b,                                /* Analog Devices Vendor ID */
    .product_id = 0x0005,                               /* Product ID. */

    .p_unit_and_terminal_descriptors = (unsigned char*)user_audio_unit_and_terminal_descriptor,
    .unit_and_terminal_descriptors_length = sizeof(user_audio_unit_and_terminal_descriptor),

    /* Pointer to the Interface parameters for the Audio Stream Rx interface.
       Set to CLD_NULL if not required.*/
    .p_audio_streaming_rx_interface_params = &user_audio_out_endpoint_params,

    /* Pointer to the Interface parameters for the Audio Stream Tx interface.
       Set to CLD_NULL if not required.*/
    .p_audio_streaming_tx_interface_params = &user_audio_in_endpoint_params,

    /* This function is called when the data is received on the Isochronous OUT endpoint.
        If this function returns an error it causes the next Isochronous OUT packet to be stalled. */
    .fp_audio_stream_data_received = user_audio_stream_data_received,

    /* Function called when an USB Audio 1.0 Set Request is received. The User code should set
       p_transfer_data->p_data_buffer to address where to store the data for the command
       described by p_req_params. */
    .fp_audio_set_req_cmd = user_audio_set_req_cmd,

    /* Function called when an USB Audio 1.0 Get Request is received.
       The User code should set p_transfer_data->p_data_buffer and
       p_transfer_data->num_bytes to describe the data to return to the Host */
    .fp_audio_get_req_cmd = user_audio_get_req_cmd,

    /* Function called when the Isochronous OUT endpoint interface is enabled/disabled by
       the USB Host using the Set Interface Alternate Setting parameter. */
    .fp_audio_streaming_rx_endpoint_enabled = user_audio_streaming_rx_endpoint_enabled,
    /* Function called when the Isochronous IN endpoint interface is enabled/disabled by
       the USB Host using the Set Interface Alternate Setting parameter. */
    .fp_audio_streaming_tx_endpoint_enabled = user_audio_streaming_tx_endpoint_enabled,


    .usb_bus_max_power = 0,                             /* The ADSP-BF707 EZ-Board is
                                                           self-powered (not USB bus-powered) */

    .device_descriptor_bcdDevice = 0x0100,              /* Set USB Device Descriptor
                                                           firmware version to 1.00 */
    /* USB string descriptors - Set to CLD_NULL if not required */
    .p_usb_string_manufacturer  = "Analog Devices Inc",
    .p_usb_string_product       = "BF707 Audio v1.0 Device",
    .p_usb_string_serial_number = CLD_NULL,
    .p_usb_string_configuration = CLD_NULL,
    .p_usb_string_audio_control_interface = CLD_NULL,
    .p_usb_string_audio_streaming_out_interface = CLD_NULL,
    .p_usb_string_audio_streaming_in_interface = CLD_NULL,

    .user_string_descriptor_table_num_entries = 0,
    .p_user_string_descriptor_table = CLD_NULL,

    .usb_string_language_id     = 0x0409,               /* English (US) language ID */

    /* Function called when one of the following USB events occurs
        CLD_USB_CABLE_CONNECTED      - USB cable connected
        CLD_USB_CABLE_DISCONNECTED   - USB cable disconnected
        CLD_USB_ENUMERATED_CONFIGURED- USB Enumerated (USB Configuration set to non-zero value)
        CLD_USB_UN_CONFIGURED        - USB Configuration set to 0.
        CLD_USB_BUS_RESET            - USB Bus Reset received. */
    .fp_cld_usb_event_callback = user_audio_usb_event,
};

/* USB Volume Levels */
typedef struct
{
    unsigned short master;
    unsigned short left;
    unsigned short right;
    unsigned short min;
    unsigned short max;
    unsigned short resolution;
    unsigned char mute;
} User_Audio_Data_Volume;

typedef struct
{
    unsigned short left;
    unsigned short right;
    unsigned char mute;
} User_Audio_Req_Volume;

/* Circular Queue used to buffer Codec Microphone data until it can be sent to the USB Host */
#define USER_AUDIO_CODEC_QUEUE_NUM_BUFFERS  3
typedef struct
{
    unsigned char head;
    unsigned char tail;
    unsigned char free_space;
    struct
    {
        unsigned short length;
        unsigned char buffer[USER_AUDIO_MAX_PACKET_SIZE];
    } data[USER_AUDIO_CODEC_QUEUE_NUM_BUFFERS];
} User_Audio_Codec_Data_Queue;

/* User Audio data */
typedef struct
{
    CLD_Boolean             volume_changed;                 /* CLD_TRUE when a Set Volume or
                                                               Mute command is received */
    User_Audio_Data_Volume  headphone_output_volume;        /* Current Volume settings */
    User_Audio_Req_Volume   headphone_req_output_volume;    /* Requested Volume settings */
    User_Audio_Data_Volume  mic_input_volume;               /* Current Microphone Volume Settings */
    unsigned short          microphone_req_input_volume;    /* Requested Microphone Volume */
                                                            /* Microphone Tx data queue */
    User_Audio_Codec_Data_Queue mic_data_queue;
                                                            /* Headphone audio data Rx buffer */
    unsigned char           headphone_data_rx_buffer[USER_AUDIO_MAX_PACKET_SIZE];
    unsigned short cur_stream_out_transfer_size;            /* Number of bytes in
                                                               headphone_data_rx_buffer. */
    CLD_Boolean isochronous_in_enabled;                     /* CLD_TRUE - Isochronous IN endpoint enabled */
    CLD_Boolean isochronous_in_idle;                        /* CLD_TRUE - Endpoint is idle so data can
                                                                          be transmitted. */
} User_Audio_Data;

static User_Audio_Data user_audio_data =
{
    .volume_changed = CLD_FALSE,
    .headphone_output_volume.min = 0xc700,                  /* ADAU1761 USB Minimum Volume Setting = -57dB */
    .headphone_output_volume.max = 0x0600,                  /* ADAU1761 USB Maximum Volume Setting = 6dB */
    .headphone_output_volume.resolution = 0x0100,           /* ADAU1761 USB Volume Resolution Setting = 1dB */

    .mic_input_volume.min = 0xa0a0,                         /* ADAU1761 USB Minimum Volume Setting = -95.3750dB */
    .mic_input_volume.max = 0x0000,                         /* ADAU1761 USB Maximum Volume Setting = 0dB */
    .mic_input_volume.resolution = 0x0060,                  /* ADAU1761 USB Volume Resolution Setting = 0.3750dB */

    /* Initialize the microphone tx data circular queue */
    .mic_data_queue.head = 0,
    .mic_data_queue.tail = 0,
    .mic_data_queue.free_space = USER_AUDIO_CODEC_QUEUE_NUM_BUFFERS,

    .isochronous_in_enabled = CLD_FALSE,
    .isochronous_in_idle = CLD_TRUE,
};


/*=============================================================================
Function:       user_audio_init

Parameters:     None.

Description:    Initializes the CLD Bulk library.

Returns:        USER_AUDIO_INIT_SUCCESS/USER_AUDIO_INIT_ONGOING/USER_AUDIO_INIT_FAILED.
==============================================================================*/
User_Audio_Init_Return_Code user_audio_init (void)
{
    static unsigned char user_init_state = 0;
    CLD_RV cld_rv = CLD_ONGOING;

    /* Initialize the CLD Audio v1.0 Library */
    cld_rv = cld_bf70x_audio_1_0_lib_init(&user_audio_init_params);

    if (cld_rv == CLD_SUCCESS)
    {
        /* Get the current volume settings for Headphone Left and Right channels */
        user_adau1761_GetVolume (hADAU1761,
                                ADI_ADAU1761_VOL_HEADPHONE,
                                ADI_ADAU1761_VOL_CHAN_LEFT,
                                &user_audio_data.headphone_output_volume.left);

        user_adau1761_GetVolume (hADAU1761,
                                ADI_ADAU1761_VOL_HEADPHONE,
                                ADI_ADAU1761_VOL_CHAN_RIGHT,
                                &user_audio_data.headphone_output_volume.right);

        /* Get the current Mute settings for Headphones */
        user_audio_data.headphone_output_volume.mute = user_adau1761_GetMute(hADAU1761,
                                ADI_ADAU1761_VOL_HEADPHONE,
                                ADI_ADAU1761_VOL_CHAN_LEFT);

        /* Get the current volume setting for microphone input */
        user_adau1761_GetVolume (hADAU1761,
                                ADI_ADAU1761_VOL_ADC_MIC,
                                ADI_ADAU1761_VOL_CHAN_LEFT,
                                &user_audio_data.mic_input_volume.left);

        /* Initialize the requested volume values with the current volume settings */
        user_audio_data.headphone_req_output_volume.left = user_audio_data.headphone_output_volume.left;
        user_audio_data.headphone_req_output_volume.right = user_audio_data.headphone_output_volume.right;
        user_audio_data.headphone_req_output_volume.mute = user_audio_data.headphone_output_volume.mute;
        user_audio_data.microphone_req_input_volume = user_audio_data.mic_input_volume.left;

        /* Connect to the USB Host */
        cld_lib_usb_connect();

        return USER_AUDIO_INIT_SUCCESS;
    }
    else if (cld_rv == CLD_FAIL)
    {
        return USER_AUDIO_INIT_FAILED;
    }
    else
    {
        return USER_AUDIO_INIT_ONGOING;
    }
}

/*=============================================================================
Function:       user_audio_main

Parameters:     None.

Description:    Example user mainline.

Returns:        None.
==============================================================================*/
void user_audio_main (void)
{
    #define MINUTES(x)      (x/60000)
    #define SECONDS(x)      ((x/1000)%60)
    #define M_SECONDS(x)    (x%1000)

    static CLD_USB_Transfer_Params audio_data_tx_params;
    static CLD_Time main_time = 0;
    static CLD_Time msg_time = 0;
    static CLD_Time run_time = 0;

    cld_bf70x_audio_1_0_lib_main();

    if (cld_time_passed_ms(main_time) >= 250u)
    {
        main_time = cld_time_get();
        pADI_PORTA->DATA_TGL = (1 << 0);
    }
    if (cld_time_passed_ms(msg_time) >= 237u)
    {
        msg_time = cld_time_get();
        run_time = cld_time_passed_ms(0);
        cld_console(CLD_CONSOLE_GREEN, CLD_CONSOLE_BLACK, "Run time: %2.2d:%2.2d:%3.3d\r",
            MINUTES(run_time),
            SECONDS(run_time),
            M_SECONDS(run_time));
    }

    /* If a Set Volume or Set Mute command data has been received */
    if (user_audio_data.volume_changed == CLD_TRUE)
    {
        user_audio_data.volume_changed = CLD_FALSE;

        if (user_audio_data.headphone_req_output_volume.left != user_audio_data.headphone_output_volume.left)
        {
            user_audio_data.headphone_output_volume.left = user_audio_data.headphone_req_output_volume.left;
            user_adau1761_SetVolume(hADAU1761,
                                    ADI_ADAU1761_VOL_HEADPHONE,
                                    ADI_ADAU1761_VOL_CHAN_LEFT,
                                    user_audio_data.headphone_output_volume.left);
        }
        if (user_audio_data.headphone_req_output_volume.right != user_audio_data.headphone_output_volume.right)
        {
            user_audio_data.headphone_output_volume.right = user_audio_data.headphone_req_output_volume.right;
            user_adau1761_SetVolume(hADAU1761,
                                    ADI_ADAU1761_VOL_HEADPHONE,
                                    ADI_ADAU1761_VOL_CHAN_RIGHT,
                                    user_audio_data.headphone_output_volume.right);
        }
        if (user_audio_data.headphone_req_output_volume.mute != user_audio_data.headphone_output_volume.mute)
        {
            user_audio_data.headphone_output_volume.mute = user_audio_data.headphone_req_output_volume.mute;
            user_adau1761_SetMute (hADAU1761, ADI_ADAU1761_VOL_HEADPHONE,
                                   user_audio_data.headphone_output_volume.mute);
        }

        if (user_audio_data.microphone_req_input_volume != user_audio_data.mic_input_volume.left)
        {
            user_audio_data.mic_input_volume.left = user_audio_data.microphone_req_input_volume;
            user_adau1761_SetVolume(hADAU1761,
                                    ADI_ADAU1761_VOL_ADC_MIC,
                                    ADI_ADAU1761_VOL_CHAN_BOTH,
                                    user_audio_data.mic_input_volume.left);
        }
    }

    /* If the Isochronous IN endpoint is enabled and idle */
    if ((user_audio_data.isochronous_in_enabled == CLD_TRUE) &&
        (user_audio_data.isochronous_in_idle == CLD_TRUE))
    {
        /* If there is headphone data waiting to be sent */
        if (user_audio_data.mic_data_queue.free_space != USER_AUDIO_CODEC_QUEUE_NUM_BUFFERS)
        {
            /* Transmit the Headphone data using the Audio Streaming IN interface from the tail of the microphone
               circular queue */
            audio_data_tx_params.num_bytes = user_audio_data.mic_data_queue.data[user_audio_data.mic_data_queue.tail].length;
            audio_data_tx_params.p_data_buffer = user_audio_data.mic_data_queue.data[user_audio_data.mic_data_queue.tail].buffer;
            audio_data_tx_params.callback.fp_usb_in_transfer_complete = user_audio_audio_stream_data_transmitted;
            audio_data_tx_params.transfer_timeout_ms = 1000;
            audio_data_tx_params.fp_transfer_aborted_callback = user_audio_audio_stream_data_transmit_aborted;
            if (cld_bf70x_audio_1_0_lib_transmit_audio_data (&audio_data_tx_params) == CLD_USB_TRANSMIT_SUCCESSFUL)
            {
                user_audio_data.isochronous_in_idle = CLD_FALSE;
            }
        }
    }
}

/*=============================================================================
Function:       user_audio_audio_stream_data_transmitted

Parameters:     None.

Description:    Function called when a microphone data packet has been transmitted
                to the USB Host.

Returns:        None.
==============================================================================*/
static void user_audio_audio_stream_data_transmitted (void)
{
    /* Advance the tail of the microphone data circular queue */
    user_audio_data.mic_data_queue.tail++;
    if (user_audio_data.mic_data_queue.tail >= USER_AUDIO_CODEC_QUEUE_NUM_BUFFERS)
    {
        user_audio_data.mic_data_queue.tail = 0;
    }
    user_audio_data.mic_data_queue.free_space++;

    user_audio_data.isochronous_in_idle = CLD_TRUE;
}

/*=============================================================================
Function:       user_audio_audio_stream_data_transmit_aborted

Parameters:     None.

Description:    Function called when a the microphone data transmission fails.

Returns:        None.
==============================================================================*/
static void user_audio_audio_stream_data_transmit_aborted (void)
{
    user_audio_data.isochronous_in_idle = CLD_TRUE;
}

/*=============================================================================
Function:       user_audio_stream_data_received

Parameters:     p_transfer_data - Pointer to the Isochronous OUT transfer data.
                p_transfer_data->num_bytes     - Number of received OUT bytes.
                                                 This value can be set to the total
                                                 transfer size if more then one
                                                 packet is expected.
                p_transfer_data->p_data_buffer - Set to the address where the
                                                 data should be written.
                p_transfer_data->callback.fp_usb_out_transfer_complete -
                                                 Function called when the
                                                 requested received bytes have
                                                 been written to p_data_buffer.
                p_transfer_data->fp_transfer_aborted_callback -
                                                 Optional function that is
                                                 called if the transfer is
                                                 aborted.
                p_transfer_data->transfer_timeout_ms - Transfer timeout in milliseconds.
                                                 Set to 0 to disable timeout.

Description:    This function is called by the cld_bf70x_audio_1_0 library when data is
                received on the Isochronous OUT endpoint. This function sets the
                p_transfer_data parameters to select where the received data
                should be stored, and what function should be called when the
                transfer is complete.

Returns:        CLD_USB_TRANSFER_ACCEPT - Store the data using the p_transfer_data
                                          parameters.
                CLD_USB_TRANSFER_PAUSE - The device isn't ready to process this
                                         out packet so pause the transfer
                                         until the cld_bf70x_audio_1_0_lib_resume_paused_audio_data_transfer
                                         function is called.
                CLD_USB_TRANSFER_DISCARD - Discard this packet.
                CLD_USB_TRANSFER_STALL - Stall the OUT endpoint.
==============================================================================*/
static CLD_USB_Transfer_Request_Return_Type user_audio_stream_data_received (CLD_USB_Transfer_Params * p_transfer_data)
{
    user_audio_data.cur_stream_out_transfer_size = p_transfer_data->num_bytes;
    p_transfer_data->p_data_buffer = user_audio_data.headphone_data_rx_buffer;
    p_transfer_data->transfer_timeout_ms = 0;
    p_transfer_data->fp_transfer_aborted_callback = CLD_NULL;
    p_transfer_data->callback.fp_usb_out_transfer_complete = user_audio_stream_data_receive_complete;

    return CLD_USB_TRANSFER_ACCEPT;
}

/*=============================================================================
Function:       user_audio_stream_data_receive_complete

Parameters:     None.

Description:    Function called when headphone data has been received.

Returns:        None.
==============================================================================*/
static CLD_USB_Data_Received_Return_Type user_audio_stream_data_receive_complete (void)
{
    static User_ADAU1761_Audio_Transfer_Params adau1761_transfer_params;

    adau1761_transfer_params.p_buffer = user_audio_data.headphone_data_rx_buffer;
    adau1761_transfer_params.buffer_size = user_audio_data.cur_stream_out_transfer_size;
    adau1761_transfer_params.fp_xfr_complete = user_audio_adau1761_buffer_transmitted;

    /* Pass the received headphone data to the adau1761 Codec */
    user_adau1761_transfer_audio_buffer(&adau1761_transfer_params);
    return CLD_USB_DATA_GOOD;
}

/*=============================================================================
Function:       user_audio_adau1761_buffer_transmitted

Parameters:     None.

Description:    Function called when headphone data has been transmitted to the
                ADAU1761 codec.

Returns:        None.
==============================================================================*/
static void user_audio_adau1761_buffer_transmitted (void)
{
    static unsigned char cnt = 0;

    /* Toggle LED 5 on the BF707 EZ-Board */
    cnt++;
    if (cnt == 75)
    {
           cnt = 0;
           pADI_PORTB->DATA_TGL = (1 << 1);
    }
}

/*=============================================================================
Function:       user_audio_set_req_cmd

Parameters:     p_transfer_data - Pointer to the Control OUT transfer data.
                p_transfer_data->num_bytes     - Number of Control OUT bytes.
                p_transfer_data->p_data_buffer - Set to the address where to
                                                 store the Control OUT data.
                p_transfer_data->callback.usb_in_transfer_complete -
                                                 Function called when the
                                                 Control OUT bytes have been
                                                 received.
                p_transfer_data->transfer_aborted_callback -
                                                 Optional function that is
                                                 called if the transfer is
                                                 aborted.
                p_req_params->req              - Setup Packet bRequest value.
                p_req_params->entity_id        - Requested entity ID (Unit ID,
                                                 Terminal ID, etc)
                p_req_params->interface_or_endpoint_num -
                                                 Requested interface or endpoint
                                                 number depending on the entity.
                p_req_params->setup_packet_wValue

Description:    This function is called by the CLD Audio v1.0 library when a Set
                Command request is received. This function sets the
                p_transfer_data parameters to select where the received data
                should be stored, and what function should be called when the
                transfer is complete.

Returns:        CLD_USB_TRANSFER_ACCEPT - Store the data using the p_transfer_data
                                          parameters.
                CLD_USB_TRANSFER_PAUSE - The device isn't ready to process this
                                         Control OUT request so pause the transfer
                                         until the cld_bf70x_audio_1_0_lib_resume_paused_control_transfer
                                         function is called.
                CLD_USB_TRANSFER_DISCARD - Discard the received data.
                CLD_USB_TRANSFER_STALL - Stall the Control endpoint.
==============================================================================*/
static CLD_USB_Transfer_Request_Return_Type user_audio_set_req_cmd (CLD_BF70x_Audio_1_0_Cmd_Req_Parameters * p_req_params, CLD_USB_Transfer_Params * p_transfer_data)
{
    CLD_USB_Transfer_Request_Return_Type rv = CLD_USB_TRANSFER_DISCARD;

    /* If the USB Host is changing the Headphone Volume */
    if (p_req_params->entity_id == USER_AUDIO_HEADPHONE_FEATURE_UNIT_ID)
    {
        if (p_req_params->req == CLD_SET_CURRENT)
        {
            /* Left Volume */
            if (p_req_params->setup_packet_wValue == USER_AUDIO_HEADPHONE_CONTROL_AND_CHANNEL_NUM_LEFT)
            {
                /* Store the volume setting in the headphone_req_output_volume structure */
                p_transfer_data->p_data_buffer = (unsigned char *)&user_audio_data.headphone_req_output_volume.left;
                p_transfer_data->num_bytes = 2;
            }
            /* Right Volume */
            else if (p_req_params->setup_packet_wValue == USER_AUDIO_HEADPHONE_CONTROL_AND_CHANNEL_NUM_RIGHT)
            {
                /* Store the volume setting in the headphone_req_output_volume structure */
                p_transfer_data->p_data_buffer = (unsigned char *)&user_audio_data.headphone_req_output_volume.right;
                p_transfer_data->num_bytes = 2;
            }
            else /* Mute */
            {
                /* Store the mute setting in the headphone_req_output_volume structure */
                p_transfer_data->p_data_buffer = (unsigned char *)&user_audio_data.headphone_req_output_volume.mute;
                p_transfer_data->num_bytes = 1;
            }
            p_transfer_data->transfer_timeout_ms = 0;
            p_transfer_data->fp_transfer_aborted_callback = CLD_NULL;
            p_transfer_data->callback.fp_usb_out_transfer_complete = user_audio_set_volume_req;
            rv = CLD_USB_TRANSFER_ACCEPT;
        }
    }
    /* If the USB Host is changing the Microphone Volume */
    else if (p_req_params->entity_id == USER_AUDIO_MICROPHONE_FEATURE_UNIT_ID)
    {
        if (p_req_params->req == CLD_SET_CURRENT)
        {
            /* Master Volume */
            if (p_req_params->setup_packet_wValue == USER_AUDIO_MICROPHONE_CONTROL_AND_CHANNEL_NUM_MASTER)
            {
                /* Store the volume setting in the microphone_req_input_volume buffer */
                p_transfer_data->p_data_buffer = (unsigned char *)&user_audio_data.microphone_req_input_volume;
                p_transfer_data->num_bytes = 2;
                p_transfer_data->transfer_timeout_ms = 0;
                p_transfer_data->fp_transfer_aborted_callback = CLD_NULL;
                p_transfer_data->callback.fp_usb_out_transfer_complete = user_audio_set_volume_req;
                rv = CLD_USB_TRANSFER_ACCEPT;
            }
        }
    }
    return rv;
}

/*=============================================================================
Function:       user_audio_set_volume_req

Parameters:     None.

Description:    This function is called by the CLD Audio library when a Set Volume request
                data has been received.

Returns:        CLD_USB_DATA_GOOD - Received data is valid.
                CLD_USB_DATA_BAD_STALL - Received data is invalid.
==============================================================================*/
static CLD_USB_Data_Received_Return_Type user_audio_set_volume_req (void)
{
    /* Flag that new volume settings have been received. */
    user_audio_data.volume_changed = CLD_TRUE;
    return CLD_USB_DATA_GOOD;
}

/*=============================================================================
Function:       user_audio_get_req_cmd

Parameters:     p_transfer_data - Pointer to the Control IN transfer data.
                p_transfer_data->num_bytes     - Number of Control IN bytes.
                p_transfer_data->p_data_buffer - Set to the address where to
                                                 source the Control IN data.
                p_transfer_data->callback.usb_in_transfer_complete -
                                                 Function called when the
                                                 Control IN bytes have been
                                                 sent.
                p_transfer_data->transfer_aborted_callback -
                                                 Optional function that is
                                                 called if the transfer is
                                                 aborted.
                p_req_params->req              - Setup Packet bRequest value.
                p_req_params->entity_id        - Requested entity ID (Unit ID,
                                                 Terminal ID, etc)
                p_req_params->interface_or_endpoint_num -
                                                 Requested interface or endpoint
                                                 number depending on the entity.
                p_req_params->setup_packet_wValue

Description:    This function is called by the CLD Audio v1.0 library when a Get
                Command request is received. This function sets the
                p_transfer_data parameters to select where the transmit data
                should be sourced, and what function should be called when the
                transfer is complete.

Returns:        CLD_USB_TRANSFER_ACCEPT - Store the data using the p_transfer_data
                                          parameters.
                CLD_USB_TRANSFER_PAUSE - The device isn't ready to process this
                                         Control OUT request so pause the transfer
                                         until the cld_bf70x_audio_1_0_lib_resume_paused_control_transfer
                                         function is called.
                CLD_USB_TRANSFER_DISCARD - Discard the received data.
                CLD_USB_TRANSFER_STALL - Stall the Control endpoint.
==============================================================================*/
static CLD_USB_Transfer_Request_Return_Type user_audio_get_req_cmd (CLD_BF70x_Audio_1_0_Cmd_Req_Parameters * p_req_params, CLD_USB_Transfer_Params * p_transfer_data)
{
    CLD_USB_Transfer_Request_Return_Type rv = CLD_USB_TRANSFER_DISCARD;

    /* If the USB Host is Requesting the Headphone Volume */
    if (p_req_params->entity_id == USER_AUDIO_HEADPHONE_FEATURE_UNIT_ID)
    {
        /* Current Setting */
        if (p_req_params->req == CLD_GET_CURRENT)
        {
            /* Left Volume */
            if (p_req_params->setup_packet_wValue == USER_AUDIO_HEADPHONE_CONTROL_AND_CHANNEL_NUM_LEFT)
            {
                p_transfer_data->p_data_buffer = (unsigned char *)&user_audio_data.headphone_output_volume.left;
                p_transfer_data->num_bytes = 2;
            }
            /* Right Volume */
            else if (p_req_params->setup_packet_wValue == USER_AUDIO_HEADPHONE_CONTROL_AND_CHANNEL_NUM_RIGHT)
            {
                p_transfer_data->p_data_buffer = (unsigned char *)&user_audio_data.headphone_output_volume.right;
                p_transfer_data->num_bytes = 2;
            }
            else /* Mute */
            {
                p_transfer_data->p_data_buffer = (unsigned char *)&user_audio_data.headphone_output_volume.mute;
                p_transfer_data->num_bytes = 1;
            }

            p_transfer_data->transfer_timeout_ms = 0;
            p_transfer_data->fp_transfer_aborted_callback = CLD_NULL;
            p_transfer_data->callback.fp_usb_in_transfer_complete = CLD_NULL;
            rv = CLD_USB_TRANSFER_ACCEPT;
        }
        /* Minimum Headphone Settings */
        else if (p_req_params->req == CLD_GET_MIN)
        {
            p_transfer_data->p_data_buffer = (unsigned char *)&user_audio_data.headphone_output_volume.min;
            p_transfer_data->num_bytes = 2;
            p_transfer_data->transfer_timeout_ms = 0;
            p_transfer_data->fp_transfer_aborted_callback = CLD_NULL;
            p_transfer_data->callback.fp_usb_in_transfer_complete = CLD_NULL;
            rv = CLD_USB_TRANSFER_ACCEPT;
        }
        /* Maximum Headphone Settings */
        else if (p_req_params->req == CLD_GET_MAX)
        {
            p_transfer_data->p_data_buffer = (unsigned char *)&user_audio_data.headphone_output_volume.max;
            p_transfer_data->num_bytes = 2;
            p_transfer_data->transfer_timeout_ms = 0;
            p_transfer_data->fp_transfer_aborted_callback = CLD_NULL;
            p_transfer_data->callback.fp_usb_in_transfer_complete = CLD_NULL;
            rv = CLD_USB_TRANSFER_ACCEPT;
        }
        /* Headphone Volume Resolution */
        else if (p_req_params->req == CLD_GET_RESOLUTION)
        {
            p_transfer_data->p_data_buffer = (unsigned char *)&user_audio_data.headphone_output_volume.resolution;
            p_transfer_data->num_bytes = 2;
            p_transfer_data->transfer_timeout_ms = 0;
            p_transfer_data->fp_transfer_aborted_callback = CLD_NULL;
            p_transfer_data->callback.fp_usb_in_transfer_complete = CLD_NULL;
            rv = CLD_USB_TRANSFER_ACCEPT;
        }
    }
    /* If the USB Host is Requesting the Microphone Volume */
    else if (p_req_params->entity_id == USER_AUDIO_MICROPHONE_FEATURE_UNIT_ID)
    {
        /* Current microphone Volume */
        if (p_req_params->req == CLD_GET_CURRENT)
        {
            if (p_req_params->setup_packet_wValue == USER_AUDIO_MICROPHONE_CONTROL_AND_CHANNEL_NUM_MASTER) /* Master Volume */
            {
                p_transfer_data->p_data_buffer = (unsigned char *)&user_audio_data.mic_input_volume.left;
                p_transfer_data->num_bytes = 2;

                p_transfer_data->transfer_timeout_ms = 0;
                p_transfer_data->fp_transfer_aborted_callback = CLD_NULL;
                p_transfer_data->callback.fp_usb_in_transfer_complete = CLD_NULL;
                rv = CLD_USB_TRANSFER_ACCEPT;
            }
        }
        /* Minimum microphone setting */
        else if (p_req_params->req == CLD_GET_MIN)
        {
            p_transfer_data->p_data_buffer = (unsigned char *)&user_audio_data.mic_input_volume.min;
            p_transfer_data->num_bytes = 2;
            p_transfer_data->transfer_timeout_ms = 0;
            p_transfer_data->fp_transfer_aborted_callback = CLD_NULL;
            p_transfer_data->callback.fp_usb_in_transfer_complete = CLD_NULL;
            rv = CLD_USB_TRANSFER_ACCEPT;
        }
        /* Maximum microphone setting */
        else if (p_req_params->req == CLD_GET_MAX)
        {
            p_transfer_data->p_data_buffer = (unsigned char *)&user_audio_data.mic_input_volume.max;
            p_transfer_data->num_bytes = 2;
            p_transfer_data->transfer_timeout_ms = 0;
            p_transfer_data->fp_transfer_aborted_callback = CLD_NULL;
            p_transfer_data->callback.fp_usb_in_transfer_complete = CLD_NULL;
            rv = CLD_USB_TRANSFER_ACCEPT;
        }
        /* Microphone volume resolution */
        else if (p_req_params->req == CLD_GET_RESOLUTION)
        {
            p_transfer_data->p_data_buffer = (unsigned char *)&user_audio_data.mic_input_volume.resolution;
            p_transfer_data->num_bytes = 2;
            p_transfer_data->transfer_timeout_ms = 0;
            p_transfer_data->fp_transfer_aborted_callback = CLD_NULL;
            p_transfer_data->callback.fp_usb_in_transfer_complete = CLD_NULL;
            rv = CLD_USB_TRANSFER_ACCEPT;
        }
    }
    return rv;
}

/*=============================================================================
Function:       user_audio_streaming_rx_endpoint_enabled

Parameters:     enabled - CLD_TRUE = Isochronous OUT endpoint Enabled.

Description:    Function called when the Isochronous OUT interface is
                enabled/disabled by the USB Host using the Set Interface Alternate
                Setting request.

Returns:        None.
==============================================================================*/
static void user_audio_streaming_rx_endpoint_enabled (CLD_Boolean enabled)
{
    cld_console(CLD_CONSOLE_YELLOW, CLD_CONSOLE_BLACK, "Streaming OUT endpoint %sabled\n\r",(enabled == CLD_TRUE ? "en" : "dis"));

    if (enabled == CLD_TRUE)
    {
        user_adau1761_enable_output();
    }
    else
    {
        user_adau1761_disable_output();
    }
}

/*=============================================================================
Function:       user_audio_streaming_tx_endpoint_enabled

Parameters:     enabled - CLD_TRUE = Isochronous IN endpoint Enabled.

Description:    Function called when the Isochronous IN interface is
                enabled/disabled by the USB Host using the Set Interface Alternate
                Setting request.

Returns:        None.
==============================================================================*/
static void user_audio_streaming_tx_endpoint_enabled (CLD_Boolean enabled)
{
    cld_console(CLD_CONSOLE_YELLOW, CLD_CONSOLE_BLACK, "Streaming IN endpoint %sabled\n\r",(enabled == CLD_TRUE ? "en" : "dis"));

    user_audio_data.isochronous_in_enabled = enabled;

    if (enabled == CLD_TRUE)
    {
        user_adau1761_enable_input();
    }
    else
    {
        user_adau1761_disable_input();
        /* Clear the microphone data circular queue. */
        user_audio_data.mic_data_queue.head = 0;
        user_audio_data.mic_data_queue.tail = 0;
        user_audio_data.mic_data_queue.free_space = USER_AUDIO_CODEC_QUEUE_NUM_BUFFERS;

        user_audio_data.isochronous_in_idle = CLD_TRUE;
    }
}

/*=============================================================================
Function:       user_audio_console_rx_byte

Parameters:     byte - received byte.

Description:    Function called when the cld_console UART receives a byte.

Returns:        None.
==============================================================================*/
static void user_audio_console_rx_byte (unsigned char byte)
{
    static const CLD_CONSOLE_COLOR cld_console_colors[CLD_CONSOLE_NUM_COLORS] =
    {
        CLD_CONSOLE_RED,
        CLD_CONSOLE_GREEN,
        CLD_CONSOLE_YELLOW,
        CLD_CONSOLE_BLUE,
        CLD_CONSOLE_PURPLE,
        CLD_CONSOLE_CYAN,
        CLD_CONSOLE_WHITE,
        CLD_CONSOLE_BLACK,
    };
    static const char * color_strings[CLD_CONSOLE_NUM_COLORS] =
    {
        "Red",
        "Green",
        "Yellow",
        "Blue",
        "Purple",
        "Cyan",
        "White",
        "Black",
    };
    unsigned char i;

    cld_console(CLD_CONSOLE_GREEN, CLD_CONSOLE_BLACK, "\r\nUser received %c\r\n", byte);

    if (byte == '!')
    {
        cld_console(CLD_CONSOLE_GREEN, CLD_CONSOLE_BLACK, "\n\r----- CLD Console Color Demo -----\n\r\n\r");
        for (i = 0; i < CLD_CONSOLE_NUM_COLORS; i++)
        {
            cld_console(cld_console_colors[i], cld_console_colors[CLD_CONSOLE_NUM_COLORS - i - 1], "%s text with a %s background\n\r",
                        color_strings[i], color_strings[CLD_CONSOLE_NUM_COLORS - i - 1]);
        }
        cld_console(CLD_CONSOLE_GREEN, CLD_CONSOLE_BLACK, "\n\r----------------------------------\n\r\n\r");
    }
}

/*=============================================================================
Function:       user_audio_usb_event

Parameters:     event - identifies which USB event has occurred.

Description:    Function Called when a USB event occurs.

Returns:        None.
==============================================================================*/
static void user_audio_usb_event (CLD_USB_Event event)
{
    switch (event)
    {
        case CLD_USB_CABLE_CONNECTED:
            pADI_PORTA->DATA_SET = (1 << 1);
        break;
        case CLD_USB_CABLE_DISCONNECTED:
            pADI_PORTA->DATA_CLR = (1 << 1);
        break;
        case CLD_USB_ENUMERATED_CONFIGURED:
            cld_console(CLD_CONSOLE_GREEN, CLD_CONSOLE_BLACK, "CLD Audio Device Enumerated\n\r");
        break;
        case CLD_USB_UN_CONFIGURED:
        break;
        case CLD_USB_BUS_RESET:
            user_audio_streaming_tx_endpoint_enabled(CLD_FALSE);
            user_audio_streaming_rx_endpoint_enabled(CLD_FALSE);
        break;
    }
}

/*=============================================================================
Function:       user_audio_transmit_audio_stream_data

Parameters:     p_data - Pointer to the data being added to the microphone data queue
                data_length - microphone data length.

Description:    Added the data buffer passed to the function to the head of the
                microphone circular queue.

Returns:        None.
==============================================================================*/
void user_audio_transmit_audio_stream_data (unsigned char * p_data, unsigned short data_length)
{
    if (user_audio_data.isochronous_in_enabled == CLD_TRUE)
    {
        if (user_audio_data.mic_data_queue.free_space)
        {
            memcpy(user_audio_data.mic_data_queue.data[user_audio_data.mic_data_queue.head].buffer, p_data, data_length);
            user_audio_data.mic_data_queue.data[user_audio_data.mic_data_queue.head].length = data_length;

            user_audio_data.mic_data_queue.head++;
            if (user_audio_data.mic_data_queue.head >= USER_AUDIO_CODEC_QUEUE_NUM_BUFFERS)
            {
                user_audio_data.mic_data_queue.head = 0;
            }
            user_audio_data.mic_data_queue.free_space--;
        }
    }
}
